function spotClassifyByDistance(aImarisApplicationID)
% Classifies the Spots by distance from the center of a surface
%
% The distance limits are specified by the user via input dialogs. The
% classified spots are copied to new Spots objects.
%
%    <CustomTools> 
%      <Menu> 
%        <Submenu name="ImarisXT course"> 
%          <Item name="Classify spots by distance" icon="Matlab"> 
%            <Command>MatlabXT::spotClassifyByDistance(%i)</Command> 
%          </Item> 
%        </Submenu>
%      </Menu> 
%      <SurpassTab> 
%        <SurpassComponent name="bpSpots"> 
%          <Item name="Classify spots by distance" icon="Matlab"> 
%            <Command>MatlabXT::spotClassifyByDistance(%i)</Command> 
%          </Item> 
%        </SurpassComponent> 
%      </SurpassTab> 
%    </CustomTools> 
%
% Copyright (c) 2012, Aaron Ponti

% Set up connection between Imaris and MATLAB
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% Get the currently selected object in the surpass scene and check that it
% is a Spots object
vSpots = vImarisApplication.GetSurpassSelection();
vSpots = vImarisApplication.GetFactory().ToSpots(vSpots);
if isempty(vSpots)
    errordlg('Please select a Spots object!');
    return
end

% Get the Surfaces objects and ask the user to pick one if there are more
% than one
surfaces = {}; nSurfaces = 0;
nChildren = vImarisApplication.GetSurpassScene().GetNumberOfChildren();
for i = 0 : (nChildren - 1)
    child = vImarisApplication.GetSurpassScene.GetChild( i );
    if vImarisApplication.GetFactory().IsSurfaces(child)
        nSurfaces = nSurfaces + 1;
        % We must cast the child to a Surfaces object!
        surfaces{nSurfaces} = ...
            vImarisApplication.GetFactory().ToSurfaces(child);
    end
end
if nSurfaces == 0
        errordlg('There are no Surfaces in the scene!');
    return
end
if nSurfaces > 1
    % Ask the user to pick one surface
    vSurface = askUserToPickSurface(surfaces);
    if isempty(vSurface)
        return
    end
else
    vSurface = surfaces{ 1 };
end

% Get the center of mass of the Surfaces object
centerOfMass = vSurface.GetCenterOfMass(0);

% Ask the user to specify the distance limits to use
distanceLimits = askUserToSetDistanceLimits();
if isempty(distanceLimits)
    return
end

% Get the spot coordinates, radii and time indices
spotValues = vSpots.Get();
coords = spotValues.mPositionsXYZ;
radii = spotValues.mRadii;
timeIndices = spotValues.mIndicesT;

% Calculate all spot distances from the measurement point
D = sqrt(...
    (coords(:, 1) - centerOfMass(1)).^2 + ...
    (coords(:, 2) - centerOfMass(2)).^2 + ...
    (coords(:, 3) - centerOfMass(3)).^2);

% Now classify the spots by distance and create new Spots objects
% containing the spots that fall in current distance bin
distanceLimits = [0 distanceLimits Inf];
for i = 2 : numel(distanceLimits)
    indx = find(D >= distanceLimits(i - 1) & D < distanceLimits(i));
    if ~isempty(indx)
        newSpots = vImarisApplication.GetFactory().CreateSpots();
        newSpots.Set(coords(indx, :), timeIndices(indx), radii(indx));
        newSpots.SetColorRGBA(uint32(randi(255, [1 3]) * [1; 256; 256^2]));
        newSpots.SetName(['Spots ', num2str(distanceLimits(i - 1)), ...
            ' <= D < ', num2str(distanceLimits(i)), ' [', ...
            num2str(numel(indx)), ']']);
        vImarisApplication.GetSurpassScene().AddChild(newSpots, -1);
    end
end

% ==== Helper functions ===================================================

% Ask the user to pick a surface from a list
function vSurface = askUserToPickSurface(surfaces)
nSurfaces = numel(surfaces);
surfaceNames = cell(1, nSurfaces);
for i = 1 : nSurfaces
    surfaceNames{i} = char(surfaces{i}.GetName());
end
[s, v] = listdlg( ...
    'Name', 'Question', ...
    'PromptString', ...
    'Choose surface from which the distances will be calculated', ...
    'SelectionMode', 'single', ...
    'ListSize', [400 300], ...
    'ListString', surfaceNames);
if v == 0
    vSurface = [];
    return;
end
vSurface = surfaces{ s };

% Ask the user to set the distance limits a surface from a list
% (we provide reasonable defaults)
function distanceLimits = askUserToSetDistanceLimits()
answer = inputdlg({'Please set distance limits:'}, 'Input', ...
    1, {'10, 20, 30, 40, 50'});
if isempty(answer)
    return;
end
distanceLimits = str2num(answer{1});
if isempty(distanceLimits)
    errordlg('Invalid distance limits!');
    distanceLimits = [];
    return;
end
distanceLimits = [0 distanceLimits Inf];

