function spotClassifyByDistanceIIC(aImarisApplicationID)
% Classifies the Spots by distance from the center of a surface using IceImarisConnector
%
% The distance limits are specified by the user via input dialogs. The
% classified spots are copied to new Spots objects.
%
%    <CustomTools> 
%      <Menu> 
%        <Submenu name="ImarisXT course"> 
%          <Item name="Classify spots by distance using IIC" icon="Matlab"> 
%            <Command>MatlabXT::spotClassifyByDistanceIIC(%i)</Command> 
%          </Item> 
%        </Submenu>
%      </Menu> 
%      <SurpassTab> 
%        <SurpassComponent name="bpSpots"> 
%          <Item name="Classify spots by distance using IIC" icon="Matlab"> 
%            <Command>MatlabXT::spotClassifyByDistanceIIC(%i)</Command> 
%          </Item> 
%        </SurpassComponent> 
%      </SurpassTab> 
%    </CustomTools> 
%
% Copyright (c) 2012, Aaron Ponti

% Set up connection between Imaris and MATLAB
conn = IceImarisConnector(aImarisApplicationID);

% Get the currently selected object in the surpass scene
vSpots = conn.getSurpassSelection();
if isempty(vSpots)
    errordlg('Please select a Spots object!');
    return
end

% Get the Surfaces objects
surfaces = conn.getAllSurpassChildren(0, 'Surfaces');
nSurfaces = numel(surfaces);
if nSurfaces == 0
        errordlg('There are no Surfaces in the scene!');
    return
end
if nSurfaces > 1
    % Ask the user to pick one surface
    vSurface = askUserToPickSurface(surfaces);
    if isempty(vSurface)
        return
    end
else
    vSurface = surfaces{ 1 };
end

% Get the center of mass of the Surfaces object
centerOfMass = vSurface.GetCenterOfMass(0);

% Ask the user to specify the distance limits to use
distanceLimits = askUserToSetDistanceLimits();
if isempty(distanceLimits)
    return
end

% Get the spot coordinates, radii and time indices
spotValues = vSpots.Get();
coords = spotValues.mPositionsXYZ;
radii = spotValues.mRadii;
timeIndices = spotValues.mIndicesT;

% Calculate all spot distances from the measurement point
D = sqrt(...
    (coords(:, 1) - centerOfMass(1)).^2 + ...
    (coords(:, 2) - centerOfMass(2)).^2 + ...
    (coords(:, 3) - centerOfMass(3)).^2);

% Now classify the spots by distance and create new Spots objects
% containing the spots that fall in current distance bin
for i = 2 : numel(distanceLimits)
    indx = find(D >= distanceLimits(i - 1) & D < distanceLimits(i));
    if ~isempty(indx)
        name = ['Spots ', num2str(distanceLimits(i - 1)), ...
            ' <= D < ', num2str(distanceLimits(i)), ' [', ...
            num2str(numel(indx)), ']'];
        conn.createAndSetSpots(coords(indx, :), timeIndices(indx), ...
            radii(indx), name, [rand(1, 3) 0]);
    end
end

% ==== Helper functions ===================================================

% Ask the user to pick a surface from a list
function vSurface = askUserToPickSurface(surfaces)
nSurfaces = numel(surfaces);
surfaceNames = cell(1, nSurfaces);
for i = 1 : nSurfaces
    surfaceNames{i} = char(surfaces{i}.GetName());
end
[s, v] = listdlg( ...
    'Name', 'Question', ...
    'PromptString', ...
    'Choose surface from which the distances will be calculated', ...
    'SelectionMode', 'single', ...
    'ListSize', [400 300], ...
    'ListString', surfaceNames);
if v == 0
    vSurface = [];
    return;
end
vSurface = surfaces{ s };

% Ask the user to set the distance limits a surface from a list
% (we provide reasonable defaults)
function distanceLimits = askUserToSetDistanceLimits()
answer = inputdlg({'Please set distance limits:'}, 'Input', ...
    1, {'10, 20, 30, 40, 50'});
if isempty(answer)
    return;
end
distanceLimits = str2num(answer{1});
if isempty(distanceLimits)
    errordlg('Invalid distance limits!');
    distanceLimits = [];
    return;
end
distanceLimits = [0 distanceLimits Inf];
