function spotPlotCoordsOnDataSetMIP(aImarisApplicationID, rowMajor)
% Plots the z-projected Spots coordinates on the dataset MIPs
%
% To keep the code short, the function plots all found Spots objects.
% No questions asked.
%
%    <CustomTools> 
%      <Menu> 
%        <Submenu name="ImarisXT course"> 
%          <Item name="Plot spot coordinates on dataset MIP (column-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIP(%i, 0)</Command> 
%          </Item> 
%          <Item name="Plot spot coordinates on dataset MIP (row-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIP(%i, 1)</Command> 
%          </Item> 
%        </Submenu>
%      </Menu> 
%      <SurpassTab> 
%        <SurpassComponent name="bpSpots"> 
%          <Item name="Plot spot coordinates on dataset MIP (column-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIP(%i, 0)</Command> 
%          </Item> 
%          <Item name="Plot spot coordinates on dataset MIP (row-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIP(%i, 1)</Command> 
%          </Item> 
%        </SurpassComponent> 
%      </SurpassTab> 
%    </CustomTools> 
%
% Copyright (c) 2012, Aaron Ponti

% Set up connection between Imaris and MATLAB
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% Set column-major by default
if nargin == 1
    rowMajor = 0;
end

if rowMajor ~= 0 && rowMajor ~= 1
    error('rowMajor must be either 0 or 1.');
end

% Get the dataset
iDataSet = vImarisApplication.GetDataSet();
if isempty(iDataSet)
    errordlg('Please load a dataset!');
    return
end

% Find all Spots objects
spots = {}; nSpots = 0;
nChildren = vImarisApplication.GetSurpassScene().GetNumberOfChildren();
for i = 0 : (nChildren - 1)
    child = vImarisApplication.GetSurpassScene.GetChild( i );
    if vImarisApplication.GetFactory().IsSpots(child)
        nSpots = nSpots + 1;
        % We must cast the child to a Spots object!
        spots{nSpots} = ...
            vImarisApplication.GetFactory().ToSpots(child);
    end
end
if nSpots == 0
    errordlg('There are no Spots in the scene!');
    return
end

% Get the dataset class
switch char(iDataSet.GetType())
    case 'eTypeUInt8',   datatype = 'uint8';
    case 'eTypeUInt16',  datatype = 'uint16';
    case 'eTypeFloat',   datatype = 'single';
    otherwise,           error('Bad value for iDataSet.GetType()');
end

% To simplify, we consider at most 3 channels (and we create an RGB image
% for display)
nChannels = iDataSet.GetSizeC();
if nChannels > 3
    nChannels = 3;
end

% Size of the output image
if rowMajor == 0
    sizeX = iDataSet.GetSizeY();
    sizeY = iDataSet.GetSizeX();
else
    sizeX = iDataSet.GetSizeX();
    sizeY = iDataSet.GetSizeY();
end

% Prepare the image (either grayscale or RGB)
if nChannels == 1
    MIP = zeros([sizeY, sizeX, 1], datatype);
else
    MIP = zeros([sizeY, sizeX, 3], datatype);
end

% Get the stack for all channels (and timepoint 0).
for i = 1 : nChannels
    
    % Allocate memory
    stack = zeros([iDataSet.GetSizeX(), iDataSet.GetSizeY(), ...
        iDataSet.GetSizeZ()], datatype);
    
    % Get the pixel data
    switch char(iDataSet.GetType())
        case 'eTypeUInt8',
            arr = iDataSet.GetDataVolumeAs1DArrayBytes(i - 1, 0);
            stack(:) = typecast(arr, 'uint8');
        case 'eTypeUInt16',
            arr = iDataSet.GetDataVolumeAs1DArrayShorts(i - 1, 0);
            stack(:) = typecast(arr, 'uint16');
        case 'eTypeFloat',
            stack(:) = iDataSet.GetDataVolumeAs1DArrayFloats(i - 1, 0);
        otherwise,
            error('Bad value for type');
    end
    
    % Project and store
    if rowMajor == 0
        MIP(:, :, i) =  max(stack, [], 3);
    else
        MIP(:, :, i) =  (max(stack, [], 3))'; % Transposed
    end
    
end

% Display the MIP
figure; imshow(MIP, []);
hold on;

% Since we project along the z axis, we can ignore the z coordinates!
%
% Spot coordinates are stored in dataset units (usually um), therefore we
% need to map them to pixel coordinates to be able to plot them on the MIP.
% For this we need the dataset voxel sizes

% Voxel size X
voxelSizesX = (iDataSet.GetExtendMaxX() - iDataSet.GetExtendMinX()) / ...
    iDataSet.GetSizeX();

% Voxel size Y
voxelSizesY = (iDataSet.GetExtendMaxY() - iDataSet.GetExtendMinY()) / ...
    iDataSet.GetSizeY();

% Prepare some plot colors (that match our example dataset)
colors = [ 1 0 0; 0 1 0; 0 0 1];

for i = 1 : nSpots
    
    % Coords in dataset units
    coords = spots{i}.GetPositionsXYZ();
    
    % Voxels positions X
    posX = (coords(:, 1) - iDataSet.GetExtendMinX()) ./ voxelSizesX + 0.5;

    % Voxels positions Y
    posY = (coords(:, 2) - iDataSet.GetExtendMinY()) ./ voxelSizesY + 0.5;

    % Plot
    if rowMajor == 0
        plot(posY, posX, '*', 'MarkerSize', 8, 'Color', colors(i, :));
    else
        plot(posX, posY, '*', 'MarkerSize', 8, 'Color', colors(i, :));
    end
    
end
