function spotPlotCoordsOnDataSetMIPIIC(aImarisApplicationID, rowMajor)
% Plots the z-projected Spots coordinates on the dataset MIPs using IceImarisConnector
%
% To keep the code short, the function plots all found Spots objects.
% No questions asked.
%
%    <CustomTools> 
%      <Menu> 
%        <Submenu name="ImarisXT course"> 
%          <Item name="Plot spot coordinates on dataset MIP using IIC (column-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIPIIC(%i, 0)</Command> 
%          </Item> 
%          <Item name="Plot spot coordinates on dataset MIP using IIC (row-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIPIIC(%i, 1)</Command> 
%          </Item> 
%        </Submenu>
%      </Menu> 
%      <SurpassTab> 
%        <SurpassComponent name="bpSpots"> 
%          <Item name="Plot spot coordinates on dataset MIP using IIC (column-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIPIIC(%i, 0)</Command> 
%          </Item> 
%          <Item name="Plot spot coordinates on dataset MIP using IIC (row-major)" icon="Matlab"> 
%            <Command>MatlabXT::spotPlotCoordsOnDataSetMIPIIC(%i, 1)</Command> 
%          </Item> 
%        </SurpassComponent> 
%      </SurpassTab> 
%    </CustomTools> 
%
% Copyright (c) 2012, Aaron Ponti

% Set up connection between Imaris and MATLAB
conn = IceImarisConnector(aImarisApplicationID);

% Set column-major by default
if nargin == 1
    rowMajor = 0;
end

if rowMajor ~= 0 && rowMajor ~= 1
    error('rowMajor must be wither 0 or 1.');
end

% Get the dataset sizes := [sizeX sizeY sizeZ sizeC sizeT]
[sizeX sizeY sizeZ sizeC sizeT] = conn.getSizes();

% Get the dataset
if sizeX == 0
    errordlg('Please load a dataset!');
    return
end

% Find all Spots objects
spots = conn.getAllSurpassChildren(1, 'Spots');
nSpots = numel(spots);
if nSpots == 0
    errordlg('There are no Spots in the scene!');
    return
end

% Get the datatype
datatype = conn.getMatlabDatatype();

% Our display will be either grayscale (for one channel), or RGB, for any
% other number of channels
if sizeC == 1
    nChannels = 1;
else
    nChannels = 3;
end

% Prepare the image with the correct size (either grayscale or RGB)
if rowMajor == 0
    MIP = zeros([sizeX, sizeY, nChannels], datatype);
else
    MIP = zeros([sizeY, sizeX, nChannels], datatype);
end

% Get the stack for all channels (and timepoint 0).
for i = 1 : sizeC
    
    if i > nChannels
        break;
    end
    
    % Get the stack
    if rowMajor == 0
        stack = conn.getDataVolume(i - 1, 0);
    else
        stack = conn.getDataVolumeRM(i - 1, 0);
    end
    
    % Project and store
    MIP(:, :, i) =  max(stack, [], 3);
    
end

% Display the MIP
figure; imshow(MIP, []);
hold on;

% Since we project along the z axis, we can ignore the z coordinates!
%
% Spot coordinates are stored in dataset units (usually um), therefore we
% need to map them to pixel coordinates to be able to plot them on the MIP.

% Prepare some plot colors (that match our example dataset)
colors = [ 1 0 0; 0 1 0; 0 0 1];

for i = 1 : nSpots
    
    % Coords in dataset units
    coords = spots{i}.GetPositionsXYZ();
    
    % Map the coordinates to pixels
    [posX, posY] = conn.mapPositionsUnitsToVoxels(coords);
    
    % Plot
    if rowMajor == 0
        plot(posY, posX, '*', 'MarkerSize', 8, 'Color', colors(i, :));
    else
        plot(posX, posY, '*', 'MarkerSize', 8, 'Color', colors(i, :));
    end
    
end
